// Copyright (c) 2014 Dawn Robotics Ltd

// Demo showing how to use the infra red sensorson the Explorer PCB

const int NUM_IR_SENSORS = 4;
const int IR_LED_PINS[ NUM_IR_SENSORS ] = { A0, A0, A1, A1 };   
const int IR_SENSOR_PINS[ NUM_IR_SENSORS ] = { A3, A2, A4, A5 };

int readIRSensor( int ledPin, int sensorPin )
{
    const int READ_DELAY_MS = 1;

    digitalWrite( ledPin, HIGH );                // Turn on the IR LED
    delay( READ_DELAY_MS );                      // Wait a bit for the IR sensor to respond
    int totalIR = analogRead( sensorPin );       // Get total IR - reflected + background IR 
    digitalWrite( ledPin, LOW );                 // Turn off the IR LED
    delay( READ_DELAY_MS );                      // Wait a bit for the IR sensor to respond
    int backgroundIR = analogRead( sensorPin );  // Read just background IR
    int reflectedIR = totalIR - backgroundIR;    // Calculate the reflected IR

    return reflectedIR;
}

void setup()
{
    // Set all of the IR LED pins to be outputs
    for ( int i = 0; i < NUM_IR_SENSORS; i++ )
    {
        pinMode( IR_LED_PINS[ i ], OUTPUT );
    }
    
    // Open up a serial connection to send readings back
    Serial.begin( 9600 );
}

void loop()
{
    // Read from each of the IR sensors in turn
    int infraRedReadings[ NUM_IR_SENSORS ];
    
    for ( int i = 0; i < NUM_IR_SENSORS; i++ )
    {
        infraRedReadings[ i ] = readIRSensor( IR_LED_PINS[ i ], IR_SENSOR_PINS[ i ] );
    }
    
    // Send back the IR readings over the serial connection
    for ( int i = 0; i < NUM_IR_SENSORS; i++ )
    {
        Serial.print( infraRedReadings[ i ] );
        
        if ( i < NUM_IR_SENSORS - 1 )
        {
            // Not on the last IR reading so print a space
            Serial.print( " " );
        }
        else
        {
            // On the last IR reading so move onto a new line
            Serial.println( "" );
        }
    }
    
    // Wait a bit so that the IR readings to display too quickly
    delay( 500 );
        
}
