// Copyright (c) 2014 Dawn Robotics Ltd

// This program demonstrates how to use the motors of the Rover 5 robot, and how to
// structure a robot control program with a simple state machince. The robot repeatedly
// drives forwards, left, right, and backwards. Try experimenting with different values
// for ABSOLUTE_MOTOR_SPEED and STATE_TIME_MS.

enum eRobotState
{
  eRS_GoingForward,
  eRS_GoingBackward,
  eRS_TurningLeft,
  eRS_TurningRight,
  eRS_Stopped
};

const int LEFT_DIR_PIN = 12;
const int LEFT_PWM_PIN = 11;
const int RIGHT_DIR_PIN = 7;
const int RIGHT_PWM_PIN = 6;

const int STATE_TIME_MS = 1000;  // Time to spend in each state (1000ms = 1s)

const int ABSOLUTE_MOTOR_SPEED = 128;  // Choose a number from 0 (0%) to 255 (100%)

eRobotState gRobotState;
unsigned long gStateStartTimeMS;

void setState( eRobotState newState );

void setup()
{
  // Set all of the motor pins as outputs
  pinMode( LEFT_DIR_PIN, OUTPUT );
  pinMode( LEFT_PWM_PIN, OUTPUT );
  pinMode( RIGHT_DIR_PIN, OUTPUT );
  pinMode( RIGHT_PWM_PIN, OUTPUT );
  
  setState( eRS_GoingForward );
}

void loop()
{
  unsigned long curTimeMS = millis();
  if ( eRS_GoingForward == gRobotState )
  {
    digitalWrite( LEFT_DIR_PIN, LOW );
    digitalWrite( RIGHT_DIR_PIN, LOW );
    
    analogWrite( LEFT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    analogWrite( RIGHT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    
    if ( curTimeMS - gStateStartTimeMS > STATE_TIME_MS )
    {
      setState( eRS_TurningLeft );
    }
  }
  else if ( eRS_TurningLeft == gRobotState )
  {
    digitalWrite( LEFT_DIR_PIN, HIGH );
    digitalWrite( RIGHT_DIR_PIN, LOW );
    
    analogWrite( LEFT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    analogWrite( RIGHT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    
    if ( curTimeMS - gStateStartTimeMS > STATE_TIME_MS )
    {
      setState( eRS_TurningRight );
    }
  }
  else if ( eRS_TurningRight == gRobotState )
  {
    digitalWrite( LEFT_DIR_PIN, LOW );
    digitalWrite( RIGHT_DIR_PIN, HIGH );
    
    analogWrite( LEFT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    analogWrite( RIGHT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    
    if ( curTimeMS - gStateStartTimeMS > STATE_TIME_MS )
    {
      setState( eRS_Stopped );
    }
  }
  else if ( eRS_Stopped == gRobotState )
  {
    analogWrite( LEFT_PWM_PIN, 0 );
    analogWrite( RIGHT_PWM_PIN, 0 );
    
    if ( curTimeMS - gStateStartTimeMS > STATE_TIME_MS )
    {
      setState( eRS_GoingBackward );
    }
  }
  else if ( eRS_GoingBackward == gRobotState )
  {
    digitalWrite( LEFT_DIR_PIN, HIGH );
    digitalWrite( RIGHT_DIR_PIN, HIGH );
    
    analogWrite( LEFT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    analogWrite( RIGHT_PWM_PIN, ABSOLUTE_MOTOR_SPEED );
    
    if ( curTimeMS - gStateStartTimeMS > STATE_TIME_MS )
    {
      setState( eRS_GoingForward );
    }
  }
  else
  {
    // State not recognised, go back to going forwards
    setState( eRS_GoingForward );
  }
}

void setState( eRobotState newState )
{
  gRobotState = newState;
  gStateStartTimeMS = millis();
}
