// Copyright (c) 2014 Dawn Robotics Ltd

// This code has been adapted from the Ultrasonic sensor's manual which can be found at
// http://www.seeedstudio.com/wiki/Ultra_Sonic_range_measurement_module
// The basic theory of operation is that we send a short high pulse to the ultrasonic sensor
// to let it know that we want to measure distance. The ultrasonic sensor will measure
// the distance to the nearest obstacle and then send back a high pulse. The length of
// the returned high pulse corresponds to the distance to the obstacle.

const int ULTRASONIC_SENSOR_PIN = 8;
const float MAX_RANGE_IN_METRES = 4.0;

float measureRange()
{
    // Send out request pulse
    pinMode( ULTRASONIC_SENSOR_PIN, OUTPUT );
    digitalWrite( ULTRASONIC_SENSOR_PIN, LOW );
    delayMicroseconds( 2 );
    digitalWrite( ULTRASONIC_SENSOR_PIN, HIGH );
    delayMicroseconds( 5 );
    digitalWrite( ULTRASONIC_SENSOR_PIN, LOW );

    // Read in the response pulse
    pinMode( ULTRASONIC_SENSOR_PIN, INPUT );
    long durationUS = pulseIn( ULTRASONIC_SENSOR_PIN, HIGH );

    // Convert from US to metres
    long distanceCM = durationUS/29/2;
    float measuredRange = (float)distanceCM / 100.0;

    return measuredRange;
}

void setup()
{
    // Open up a serial connection to send readings back
    Serial.begin( 9600 );
}

void loop()
{
    float range = measureRange();
    
    Serial.print( "Distance = " );
    Serial.println( range );
    
    delay( 400 );
}
